"""Executa cerques en les API del LinkledIn i desa el resultat en format RDF/XML"""
# -*- coding: utf-8 -*-

import sys
import string
import time
from rdflib.namespace import Namespace
from rdflib import Graph, Literal, RDF, URIRef, BNode
import oauth2 as oauth
import xml.etree.ElementTree as ET #naveguem per un arxiu XML


__author__ = "Marc Escarmís Arasa"

class LinkedinGraph(Graph):
    """Graph de RDF"""    
    def __init__(self):
        """Inicialitza LinkedinGraph amb els Namepaces i els prefix de la ontologia"""
        super(LinkedinGraph, self).__init__()
        self.ns_linkedin = Namespace('http://www.owl-ontologies.com/Ontology1413974047.owl#')
        self.ns_rdfs = Namespace('http://www.w3.org/2000/01/rdf-schema#')
        self.bind('linkedin', self.ns_linkedin)
        self.bind('rdfs', self.ns_rdfs)


class LinkedinProcess():

    # Mètode que processa la connexió a les API del LinkedIn, i en fa
    # una primera cosulta bàsica de les feines ofertades, n'extreu 10
    def linkedin_connect(self):
        # *****************************************************************
        #   Aconsegueix les credencials necessaries per poder fer consultes a les
        #   API del LinkedIn. Ho fa mitjançant oauth2 i ussant la llibrerria de python
        #   oauth2
        #
        #   Sortida:
        #       Objecte oauth amb el  token necessari per fer consultes 
        # *****************************************************************
    
        print u'>>> Connectant amb LinkedIn '
        try:
            CONSUMER_KEY           = "7794ltf94qkh3p"
            CONSUMER_SECRET        = "QFOX4EFxrWZ4uCPU"
            OAUTH_TOKEN            = "d3973c06-c753-41bc-961b-9b77d6148346"
            OAUTH_TOKEN_SECRET     = "cb6828fa-7521-4a37-87ef-0b2452983e73"
            consumer = oauth.Consumer(CONSUMER_KEY, CONSUMER_SECRET)
            client = oauth.Client(consumer)
            
            consumer = oauth.Consumer(
                 key=CONSUMER_KEY,
                 secret=CONSUMER_SECRET)
                 
            token = oauth.Token(
                 key=OAUTH_TOKEN, 
                 secret=OAUTH_TOKEN_SECRET)

            client = oauth.Client(consumer, token)
            return(client)
        except:
            print u'>>> Possible error de connexió', sys.exc_info()[0]
            # raise

    # Creem el Graph d'RDF per anar-hi afegint les triples
    def create_graph(self):
        # ****************************************************************
        #    Crea el Graph RDF usat per la llibreria RDFlib del Python.
        #    Instancia la classe LinkedinGraph
        #
        #    Entrda:
        #
        #    Sortida:
        #        Graph RDF, varialble global
        # *****************************************************************
        
        self.graph = LinkedinGraph()
        
    # Salvem el Graph en un fitxer físic
    def save_graph(self):
        # *****************************************************************
        #    Guarda el graph en format RDF/XML
        #
        #    Entrda:
        #
        #    Sortida:
        #        Fitxer RDF/XML
        # *****************************************************************
    
        print u'>>> Desant el graph RDF en format RDF/XML'
        try:
            data = self.graph.serialize(format='xml')
            file = open('linkedin_rdf.rdf', 'w')
            file.write(data)
            file.close()
            
        except IOError:u'>>> Problemes en crear el fitxer XML per desar-hi el graf RDF'
            #raise

    # Execució de les consultes SPARQL
    def sparql_query(self,query_str):
        # *****************************************************************
        #    Executa la consulta SPARQL.
        #
        #    Entrda:
        #       query_str: Cadena amb format SPARQL
        #    Sortida:
        #       Files amb el resultat de la consulta
        # *****************************************************************
    
        response = self.graph.query(query_str)
        return(response)

    # Obtenim l'arbre XML amb un conjunt de feines        
    def get_jobs(self,client,url):
        # *****************************************************************
        #    Obté el XML com a resposta a les API del LinkedIn
        #
        #    Entrda:
        #       client: Credencials oauth2 per poder fer la consulta
        #       url: Cadena http amb la consulta
        #    Sortida:
        #       Graph RDF, varialble global
        # *****************************************************************
    
        resp, content = client.request(url)
        return(content)

    # Obtenim el subarbre XML amb les dades referents a la Companyia que ofereix la feina
    def get_company(self,client,url,company_id,subject_company,subject_job):
        # *****************************************************************
        #    Obté un subarbre XML amb la informació relativa a la companyia
        #
        #    Entrda:
        #       client: Credencials oauth2 per poder fer la consulta
        #       url: Cadena http amb la consulta
        #       company_id: identificador únic de la companyia dintra del LinkedIn
        #       subject_company: URIRef de la companyia
        #       subject_job: URIRef de la feina
        #    Sortida:
        #       Graph RDF ampliat amb la info de Companies
        # *****************************************************************
        
        resp, content = client.request(url)
        subjectBN = BNode()
        self.graph.add((subject_company, RDF.type, self.graph.ns_linkedin.Companies))
        self.graph.add((subject_company, self.graph.ns_linkedin.hasOffer, subject_job))
        self.graph.add((subject_company, self.graph.ns_linkedin.company_id, Literal(company_id)))
        self.graph.add((subject_job, self.graph.ns_linkedin.company_id, Literal(company_id)))
        sub_xml = ET.fromstring(content)
        for company in sub_xml.getchildren():
            if company.tag == 'name':
                self.graph.add((subject_company, self.graph.ns_linkedin.company_name, Literal(company.text)))
                self.graph.add((subject_job, self.graph.ns_linkedin.company_id, Literal(company_id)))
            if company.tag == 'description':
                self.graph.add((subject_company, self.graph.ns_linkedin.description, Literal(company.text)))
            if company.tag == 'universal-name':
                self.graph.add((subject_company, self.graph.ns_linkedin.universal_name, Literal(company.text)))
            if company.tag == 'num-followers':
                self.graph.add((subject_company, self.graph.ns_linkedin.num_followers, Literal(company.text)))    
            if company.tag == 'company-type':
                for code in company.iter():
                    if code.tag == 'name':
                        self.graph.add((subject_company, self.graph.ns_linkedin.company_type, Literal(code.text)))
            if company.tag == 'locations':
                for location in company.find('location'):
                    if location.tag == 'address':
                        str_adr = ''
                        for address in location.getchildren():
                            if address.tag == 'street1':
                                if address.text:
                                    str_adr = str_adr + address.text
                            if address.tag == 'city':
                                if address.text:
                                    str_adr = str_adr + ', ' + address.text
                            if address.tag == 'postal-code':
                                if address.text:
                                    str_adr = str_adr + ', ' + address.text
                self.graph.add((subject_company, self.graph.ns_linkedin.company_locations, Literal(str_adr)))    

        # print u'\n>>> Company XML\n'
        # print content
        
    # Obtenim el subarbre XML amb les dades de la Persona que ha publicat la feina
    def get_job_poster(self,client,url,poster_id,subject_poster,subject_job,subject_company):
        # *****************************************************************
        #    Obté un subarbre XML amb la informació relativa a la persona
        #
        #    Entrda:
        #       client: Credencials oauth2 per poder fer la consulta
        #       url: Cadena http amb la consulta
        #       company_id: identificador únic de la companyia dintra del LinkedIn
        #       subject_poster:  URIRef de la persona
        #       subject_company: URIRef de la companyia
        #       subject_job: URIRef de la feina
        #    Sortida:
        #       Graph RDF actualitzat amb la info de People
        # *****************************************************************
        
        resp, content = client.request(url)
        subjectBN = BNode()
        self.graph.add((subject_poster, RDF.type, self.graph.ns_linkedin.CompanyWorker))
        self.graph.add((subject_poster, self.graph.ns_linkedin.IsWorkerPoster, subject_company))
        self.graph.add((subject_poster, self.graph.ns_linkedin.IsWorkerPoster, subject_job))
        self.graph.add((subject_poster, self.graph.ns_linkedin.isApplicantOf, subjectBN))
        self.graph.add((subject_poster, self.graph.ns_linkedin.isFollowedBy, subjectBN))
        self.graph.add((subject_poster, self.graph.ns_linkedin.hasFollow, subjectBN))
        self.graph.add((subject_company, self.graph.ns_linkedin.hasWorker, subject_poster))
        self.graph.add((subject_job, self.graph.ns_linkedin.hasWorkerPoster, subject_poster))
        sub_xml = ET.fromstring(content)
        for poster in sub_xml.getchildren():
            if poster.tag == 'first-name':
                self.graph.add((subject_poster, self.graph.ns_linkedin.first_name, Literal(poster.text)))
            if poster.tag == 'last-name':
                self.graph.add((subject_poster, self.graph.ns_linkedin.last_name, Literal(poster.text)))
            if poster.tag == 'headline':
                self.graph.add((subject_poster, self.graph.ns_linkedin.headline, Literal(poster.text)))

        #print '\n>>> People XML\n'
        #print content
        
    # Processa el XML base aportat per les API de LinkedIn.
    def parse_xml(self,content):
        # *****************************************************************
        #    Processa el XML base aportat per les API de LinkedIn.
        #
        #    Entrda:
        #       content: XML retornat per les API del LinkedIn
        #       
        #    Sortida:
        #       Graph RDF actualitzat amb la info de Jobs
        # *****************************************************************
         
        print u'>>> Processant arbre XML obtingut de LinkedIn'
        subject_company = ''
        subject_job = ''
        for jobs in content.iterfind('./jobs'):
            for job in jobs.getchildren():
                if job.tag == 'job':
                    subjectBN = BNode()
                    print u'>>> Afegint informació de \'Jobs\' al Graph ',
                    for node in job.getchildren():
                        if node.tag == 'id':
                            subject_job = URIRef(self.graph.ns_linkedin['job_' + node.text])
                            print 'job_' + node.text
                            self.graph.add((subject_job, RDF.type, self.graph.ns_linkedin.Jobs))
                            self.graph.add((subject_job, self.graph.ns_linkedin.job_id, Literal(node.text)))

                        if node.tag == 'active':
                            self.graph.add((subject_job, self.graph.ns_linkedin.active, Literal(node.text)))

                        if node.tag == 'salary':
                            self.graph.add((subject_job, self.graph.ns_linkedin.salary, Literal(node.text)))

                        if node.tag == 'skills-and-experience':
                            self.graph.add((subject_job, self.graph.ns_linkedin.skills_and_experience,
                                            Literal(node.text)))

                        if node.tag == 'position':
                            for node_position in node.getchildren():
                                if node_position.tag == 'title':
                                    self.graph.add((subject_job, self.graph.ns_linkedin.position_title,
                                                    Literal(node_position.text)))

                        if node.tag == 'posting-date':
                            str_pd = ''
                            for node_pd in node.getchildren():
                                if node_pd.tag == 'year':
                                    if node_pd.text:
                                        str_pd = str_pd + node_pd.text
                                if node_pd.tag == 'month':
                                    if node_pd.text:
                                        str_pd = str_pd + '/' + node_pd.text
                                if node_pd.tag == 'day':
                                    if node_pd.text:
                                        str_pd = str_pd + '/' + node_pd.text
                            self.graph.add((subject_job, self.graph.ns_linkedin.posting_date, Literal(str_pd)))          

                        if node.tag == 'expiration-date':
                            str_pd = ''
                            for node_pd in node.getchildren():
                                if node_pd.tag == 'year':
                                    if node_pd.text:
                                        str_pd = str_pd + node_pd.text
                                if node_pd.tag == 'month':
                                    if node_pd.text:
                                        str_pd = str_pd + '/' + node_pd.text
                                if node_pd.tag == 'day':
                                    if node_pd.text:
                                        str_pd = str_pd + '/' + node_pd.text
                            self.graph.add((subject_job, self.graph.ns_linkedin.expiration_date, Literal(str_pd)))
                            
                        # Poblem la ontologia amb la informació de la 'company'. Crida al procés 'get_company' que
                        # recupera de les api de linkedin de 'Comapnies' tota la informació necessària per la ontologia.
                        if node.tag == 'company':
                            for node_company in node.getchildren():
                                if node_company.tag == 'id':
                                    company_id = node_company.text
                                    url = 'http://api.linkedin.com/v1/companies/' + node_company.text + ':' \
                                          '(id,name,description,universal-name,company-type,locations,num-followers)'
                                    subject_company = URIRef(self.graph.ns_linkedin['company_' + company_id])
                                    print u'>>> Afegint informació de \'Companies\' al Graph ',
                                    print 'company_' + company_id
                                    #fa la crida al procés per obtenir la informació de l'empresa
                                    LinkedinProcess.get_company(self,client,url,company_id,subject_company,subject_job)
                                    self.graph.add((subject_job, self.graph.ns_linkedin.isOfferedBy, subject_company))
                                    
                        # Poblem la ontologia amb la informació de 'People\CompanyWorker'.
                        # Crida al procés 'get_job_poster' que
                        # recupera de les api de linkedin de 'People' tota la informació necessària per la ontologia.
                        if node.tag == 'job-poster':
                            for node_job_poster in node.getchildren():
                                if node_job_poster.tag == 'id':
                                    if subject_company == '':
                                        subject_company = BNode()
                                    poster_id = node_job_poster.text
                                    url = 'http://api.linkedin.com/v1/people/' + node_job_poster.text + \
                                          ':(id,first-name,last-name,headline)'
                                    subject_poster = URIRef(self.graph.ns_linkedin['companyworker_' + poster_id])
                                    print u'>>> Afegint informació de \'People\' al Graph ',
                                    print 'companyworker_' + poster_id
                                    LinkedinProcess.get_job_poster(self,client,url,poster_id,subject_poster,
                                                                   subject_job,subject_company)


# *********************************************************
# **************************** MAIN ***********************
# *********************************************************

if __name__ == '__main__':

    print u'>>> Iniciant procés'
    try:
        # Instancia la classe que farà tot el procés
        linkedin_process = LinkedinProcess()
        # Obtenim les credencials per poder connectar-nos i fer consultes al LinkedIn
        client = linkedin_process.linkedin_connect()
        # Obtenim el primer arbre XML amb la informació de JOBS
        url = "https://api.linkedin.com/v1/job-search:(jobs:(id,active,posting-date,expiration-date," \
              "skills-and-experience,salary,company:(id),position:(title,location),job-poster:(id)))?count=30"
        # Processa el XML base aportat per la consulta feta a l'api de LinkedIn
        jobs = linkedin_process.get_jobs(client,url)
        linkedin_process.create_graph()
        tree = ET.fromstring(jobs)
        linkedin_process.parse_xml(tree)
        # Desa el graph
        linkedin_process.save_graph()

        # Fem una sèrie de consultes SPARQL per comprobar que s'han desat dades en l'ontologia
        print u'\n>>> Consultes SPARQL:'
        
        query_str = 'select ?h where {?h rdf:type linkedin:Jobs}'
        response = linkedin_process.sparql_query(query_str)
        print '\tNombre de Jobs desats: ' + str(len(response))
        for row in response:
            print '\t\t',
            print str(row)

        query_str = 'select ?h where {?h rdf:type linkedin:Companies}'
        response = linkedin_process.sparql_query(query_str)
        print '\tNombre de Companies desades: ' + str(len(response))
        for row in response:
            print '\t\t',
            print str(row)
            
        query_str = 'select ?h where {?h rdf:type linkedin:People}'
        response = linkedin_process.sparql_query(query_str)
        print '\tNombre de People desats: ' + str(len(response))
        for row in response:
            print "\t\t",
            print str(row)

        query_str = 'select ?h where {?h rdf:type linkedin:CompanyWorker}'
        response = linkedin_process.sparql_query(query_str)
        print '\tNombre de People-->CompanyWorker desats: ' + str(len(response))
        for row in response:
            print '\t',
            print str(row)
            
    except:
        print u'>>> Error inesperat:', sys.exc_info()[0]
        print u'>>> Comprova que la connexió a internet funciona correctament'
        # raise
